*** This is the control file for comparing the performance of       ***
***    various nonexperimental estimators of the ATT (under CIA)    ***
*** Simulation design (pla/str): placebo (Huber et al. 2013)        ***
*** DGP: theory-inspired                                            ***
*** First version: 22/06/2018                                       ***
*** This version:  18/07/2018                                       ***
*** Authors: Arun Advani, Toru Kitagawa and Tymon Sloczynski        ***
*** Written for Stata 11                                            ***


//all the usual setup stuff
*version control
version 11

*setup standard things
capture clear all
capture log close
capture graph drop _all

*set processors 1

local NAME "placebo_TS_20180718"
set seed 2018071811

*allocate memory
set mem 300m

*get rid of scroll requirement
set more off

*open log
log using `NAME', replace

*install necessary programs
capture {
	ssc install fairlie, replace
	ssc install nnmatch, replace
	ssc install oaxaca, replace
	ssc install psmatch2, replace
	ssc install moremata, replace
}


//setup locals
local LID			1 //ID of sample with lowest ID number
local HID			200 //ID of sample with highest ID number
local NREPS			1000 //number of EMCS replications per sample

local SELECT		1 //the magnitude of selection (see \lambda in Huber et al. 2010, p. 23)

local FOLDER		samples

local OUTCOME       y
local TREATMENT     treated
local COVARIATES    x1
local COV1			x1
local NCOVS			2 //number of COVARIATES + 1

*************************



#delimit ;

do common_programs;

forvalues j=`LID'/`HID' {;
	
	quietly {;
	capture erase `NAME'_all_samples_`j'.dta;
	
	quietly postfile results
		b_lr b_ipw2 ave_effect
		using `NAME'_results_`j', replace;
	
	timer on 2;
	
	use "../`FOLDER'/sample`j'", clear;
	sum `TREATMENT';
	local PRTREAT = r(mean);
	sum `TREATMENT' if `TREATMENT'==0;
	local NOBS = r(N);
		
	*estimate the 'true' propensity score;
	regress `TREATMENT' `COVARIATES';
	predict double pscore;
	predict double res, res;
	sum res; local ressd = r(sd);
	
	*control the degree of selection into treatment;
	replace pscore = pscore*`SELECT';
	
	*drop the treated;
	drop if `TREATMENT'==1;
	
	*save this file -- this is our 'population';
	save `NAME'_population, replace;
	capture drop _all;
	};
	
	forvalues i=1/`NREPS' {;
		quietly {;
		timer on 1;
		
		*******************************************DRAW A SAMPLE************************************************;
		
		use `NAME'_population;
		
		postfile MC_sample `OUTCOME' `COV1' pscore using `NAME'_sample, replace;
		
		forvalues x=1/`NOBS' {;
			gen random = uniform(); sort random; drop random;
			scalar v_`OUTCOME' = `OUTCOME'[1];
			scalar v_`COV1' = `COV1'[1];
			scalar v_pscore = pscore[1];
			post MC_sample	(v_`OUTCOME') (v_`COV1') (v_pscore);
		};
		
		postclose MC_sample; capture drop _all;
		
		*************************************SIMULATE A PLACEBO TREATMENT***************************************;
		
		use `NAME'_sample;
		
		*draw an error from the normal distribution;
		gen u = invnorm(uniform())*`ressd';
	
		replace pscore=pscore+u;

		*impose the expected proportion of treated;
		local PRTREAT2 = 100-(100*`PRTREAT');
		centile pscore, c(`PRTREAT2');
		scalar lambda = -1*r(c_1);

		*generate the treatment variable;
		gen `TREATMENT' = 1 if pscore+lambda>0;
		replace `TREATMENT' = 0 if `TREATMENT'==.;
		
		drop pscore u;
		
		save `NAME'_sample, replace;

		local state = c(seed);
		di c(seed);
		
		*****************************************RUN THE ESTIMATION*********************************************;

		*estimate propensity scores;
		regress `TREATMENT' `COVARIATES';
		predict double logps;
		
		*linear regression;
		reg `OUTCOME' `TREATMENT' `COVARIATES';
		scalar b_lr = _b[`TREATMENT'];
		
		*inverse probability weighting;
		capture ipw `OUTCOME' `TREATMENT' `COVARIATES';
		if _rc==0 {;
			scalar b_ipw2 = ipw2;
		};
		if _rc!=0 {;
			scalar b_ipw2 = .;
		};
		
		*The average treatment effect is always zero by construction;
		scalar ave_effect = 0;
		
		set seed `state';	
		
		//STORE THE OUTPUT;
		post results (b_lr) (b_ipw2) (ave_effect);
		
		timer off 1;
		noisily timer list 1;
		
		capture drop _all;
		
		capture use `NAME'_all_samples_`j', clear;
		append using `NAME'_sample;
		keep `OUTCOME' `TREATMENT' `COV1';
		save `NAME'_all_samples_`j', replace;
		};
	};
	timer clear 1;
	
	timer off 2;
	noisily display in green "Finished with sample " in yel `j';
	noisily timer list 2;
	noisily display _newline(1);
	
	postclose results;
	capture drop _all;
	erase `NAME'_sample.dta;
	erase `NAME'_population.dta;
};

log close;
