*** This is the control file for comparing the performance of       ***
***    various nonexperimental estimators of the ATT (under CIA)    ***
*** Simulation design (pla/str): structured (Busso et al. 2014)     ***
*** DGP: theory-inspired                                            ***
*** First version: 22/06/2018                                       ***
*** This version:  19/07/2018                                       ***
*** Authors: Arun Advani, Toru Kitagawa and Tymon Sloczynski        ***
*** Written for Stata 11                                            ***


//all the usual setup stuff
*version control
version 11

*setup standard things
capture clear all
capture log close
capture graph drop _all

*set processors 1

local NAME "structured_TS_20180719"
set seed 2018071925

*allocate memory
set mem 300m

*get rid of scroll requirement
set more off

*open log
log using `NAME', replace

*install necessary programs
capture {
	ssc install fairlie, replace
	ssc install nnmatch, replace
	ssc install oaxaca, replace
	ssc install psmatch2, replace
	ssc install moremata, replace
}


//setup locals
local LID			801 //ID of sample with lowest ID number
local HID			1000 //ID of sample with highest ID number
local NREPS			1000 //number of EMCS replications per sample

local NOBS			1000

local FOLDER		samples

local OUTCOME       y
local TREATMENT     treated
local COVARIATES    x1
local COV1			x1
local NCOVS			2 //number of COVARIATES + 1

*************************



#delimit ;

do common_programs;

forvalues j=`LID'/`HID' {;
	
	capture erase `NAME'_all_samples_`j'.dta;
	
	quietly postfile results
		b_lr b_ipw2 ave_effect using `NAME'_results_`j', replace;
	
	timer on 2;
	
	forvalues i=1/`NREPS' {;
		quietly {;
		timer on 1;
		
		#delimit cr
		
		use "../`FOLDER'/sample`j'", clear
		
		qui su treated if treated==0
		local NCOBS = r(N)
		
		qui su x1 if treated==0
		scalar x1_mean_c = r(mean)
		scalar x1_var_c = r(Var)
		scalar x1_max_c = r(max)
		scalar x1_min_c = r(min)
		
		qui su y if treated==0
		scalar y_max_c = r(max)
		scalar y_min_c = r(min)
		
		qui su x1 if treated==1
		scalar x1_mean_t = r(mean)
		scalar x1_var_t = r(Var)		
		scalar x1_max_t = r(max)
		scalar x1_min_t = r(min)
		
		qui su y if treated==1
		scalar y_max_t = r(max)
		scalar y_min_t = r(min)
		
		qui reg y x1 treated
		scalar ave_effect = _b[treated]
		mat y = e(b)
		predict res, r
		qui sum res, d
		scalar var_y = r(Var)
		drop res
		
		drop _all
		
		//Create new data based on the old data
		qui set obs `NOBS'
		qui gen treated = _n>`NCOBS'
		
		gen x1 = .
		replace x1 = x1_mean_c + invnormal(uniform())*sqrt(x1_var_c) if treated==0
		replace x1 = x1_mean_t + invnormal(uniform())*sqrt(x1_var_t) if treated==1
		
		replace x1 = x1_max_c if x1>x1_max_c & treated==0
		replace x1 = x1_min_c if x1<x1_min_c & treated==0
		replace x1 = x1_max_t if x1>x1_max_t & treated==1
		replace x1 = x1_min_t if x1<x1_min_t & treated==1
		
		//Generate the outcome variable
		mat score conditmean_y = y
		gen y = rnormal(conditmean_y,sqrt(var_y))
		
		replace y = y_max_c if y>y_max_c & treated==0
		replace y = y_min_c if y<y_min_c & treated==0
		replace y = y_max_t if y>y_max_t & treated==1
		replace y = y_min_t if y<y_min_t & treated==1
		
		//Keep only the necessary variables
		keep `OUTCOME' `TREATMENT' `COVARIATES'
		
		save `NAME'_sample, replace
		
		local state = c(seed)
		di c(seed)
		
		#delimit ;
		
		*****************************************RUN THE ESTIMATION*********************************************;

		*estimate propensity scores;
		regress `TREATMENT' `COVARIATES';
		predict double logps;
		
		*linear regression;
		reg `OUTCOME' `TREATMENT' `COVARIATES';
		scalar b_lr = _b[`TREATMENT'];

		*inverse probability weighting;
		capture ipw `OUTCOME' `TREATMENT' `COVARIATES';
		if _rc==0 {;
			scalar b_ipw2 = ipw2;
		};
		if _rc!=0 {;
			scalar b_ipw2 = .;
		};

		set seed `state';
		
		
		//STORE THE OUTPUT;
		post results
			(b_lr) (b_ipw2) (ave_effect);
		
		timer off 1;
		noisily timer list 1;
		
		capture drop _all;
		
		capture use `NAME'_all_samples_`j', clear;
		append using `NAME'_sample;
		keep `OUTCOME' `TREATMENT' `COVARIATES';
		save `NAME'_all_samples_`j', replace;
		};
	};
	timer clear 1;
	
	timer off 2;
	noisily display in green "Finished with sample " in yel `j';
	noisily timer list 2;
	noisily display _newline(1);
	
	postclose results;
	capture drop _all;
	erase `NAME'_sample.dta;
};

log close;
