*** Estimates using samples from the original DGP                   ***
*** First version: 30/04/2018                                       ***
*** This version:  23/05/2018                                       ***
*** Authors: Arun Advani, Toru Kitagawa and Tymon Sloczynski        ***
*** Written for Stata 11                                            ***


//all the usual setup stuff
*version control
version 11

*setup standard things
capture clear all
capture log close
capture graph drop _all

*set processors 1

*allocate memory
set mem 300m

*get rid of scroll requirement
set more off

*install necessary programs
capture {
	ssc install fairlie, replace
	ssc install nnmatch, replace
	ssc install oaxaca, replace
	ssc install psmatch2, replace
	ssc install moremata, replace
}


//setup locals

local NSAMPLES		1000 //number of samples from the original dgp
local FOLDER		samples
local NAME			2_estimates

local OUTCOME       re78
local TREATMENT     treated
local COVARIATES    age age2 age3 educ educ2 nodegree married black hispanic re74 re75 u74 u75 re74ed
local COV1			age
local COV2			age2
local COV3			age3
local COV4			educ
local COV5			educ2
local COV6			nodegree
local COV7			married
local COV8			black
local COV9			hispanic
local COV10			re74
local COV11			re75
local COV12			u74
local COV13			u75
local COV14			re74ed
local NCOVS			15 //number of COVARIATES + 1

local GRID			0.005 0.00625 0.00781 0.00977 0.01221 0.01526 0.01907 0.02384 0.0298 0.03725 0.04657 0.05821 0.07276 0.09095 0.11369

*************************



#delimit ;

quietly postfile results
	sample b_lr b_oaxaca b_dr b_kern_u b_ipw2 b_psm1 b_psmbc1 band_kern_u frac_treated N_orig N_over1 N_over2 N_T_orig N_C_orig N_T_over1 N_C_over1 N_T_over2 N_C_over2 overlap1 overlap2
	using `NAME', replace;

do "placebo/common_programs";

forvalues j=1/`NSAMPLES' {;
	timer on 1;
	quietly {;
	use "`FOLDER'/sample`j'", clear;
	
	gen u74 = 1 if re74==0;	replace u74 = 0 if re74!=0;
	gen u75 = 1 if re75==0;	replace u75 = 0 if re75!=0;
	gen u78 = 1 if re78==0; replace u78 = 0 if re78!=0;
	
	gen age2 = age^2; gen age3 = age^3;
	gen educ2 = educ^2;
	gen re742 = re74^2; gen re752 = re75^2;
	gen u74h = u74*hispanic; gen re74ed = educ*re74;
	
	
	*record the proportion of treated in this sample (and other basic measures);
	sum `TREATMENT'; scalar frac_treated = r(mean); scalar N_orig = r(N);
	sum `TREATMENT' if `TREATMENT'==1; scalar N_T_orig = r(N);
	sum `TREATMENT' if `TREATMENT'==0; scalar N_C_orig = r(N);
	
	*estimate propensity scores;
	logit `TREATMENT' `COVARIATES', asis;
	predict double logps;
	
	*this is required by kernel matching;
	sort `TREATMENT';
	
	*linear regression;
	reg `OUTCOME' `TREATMENT' `COVARIATES';
	scalar b_lr = _b[`TREATMENT'];
	
	*linear Oaxaca-Blinder;
	capture oaxaca `OUTCOME' `COVARIATES', by(`TREATMENT') weight(1);
	if _rc==0 {;
		scalar b_oaxaca = -1*_b[unexplained];
	};
	if _rc!=0 {;
		reg `OUTCOME' `COVARIATES' if `TREATMENT'==0;
		predict pred; gen diff = `OUTCOME' - pred;
		sum diff if `TREATMENT'==1; scalar diff_mean = r(mean);
		oaxaca `OUTCOME' `COVARIATES', by(`TREATMENT') weight(1) relax;
		if round(-1*_b[unexplained],.0001)==round(diff_mean,.0001) {;
		scalar b_oaxaca = -1*_b[unexplained];
		};
		else {;
		scalar b_oaxaca = .;
		};
		drop pred diff;
	};
	capture drop A1-A`NCOVS';
	
	*kernel matching (uniform);
	scalar minmsd = .; scalar minbw = .;
	foreach x of local GRID {;
		mkernel logps, tr(`TREATMENT') out(`OUTCOME') k(uniform) bw(`x') crossval(yes);
		if msd<minmsd {;
			scalar minmsd = msd; local minbw = `x';
		};
	};
	capture psmatch2 `TREATMENT', kernel out(`OUTCOME') k(uniform) p(logps) bw(`minbw');
	if _rc==0 {;
		scalar b_kern_u = r(att);
	};
	if _rc!=0 {;
		scalar b_kern_u = .;
	};
	capture drop _*; scalar band_kern_u = `minbw';
	
	*double robust regression;
	capture drobust `OUTCOME' `TREATMENT' "`COVARIATES'" no;
	if _rc==0 {;
		scalar b_dr = drob;
	};
	if _rc!=0 {;
		scalar b_dr = .;
	};
	
	*inverse probability weighting;
	capture ipw `OUTCOME' `TREATMENT' `COVARIATES';
	if _rc==0 {;
		scalar b_ipw2 = ipw2;
	};
	if _rc!=0 {;
		scalar b_ipw2 = .;
	};
	
	*matching on the estimated propensity score;
	capture nnmatch `OUTCOME' `TREATMENT' logps, tc(att) m(1);
	if _rc!=0 {;
		scalar b_psm1 = .;
	};
	if _rc==0 {;
		matrix A = e(b); svmat double A; scalar b_psm1 = A1[1]; drop A1;
	};
	
	*bias-corrected matching on the estimated propensity score;
	capture nnmatch `OUTCOME' `TREATMENT' logps, tc(att) m(1) biasadj(`COVARIATES');
	if _rc!=0 {;
		scalar b_psmbc1 = .;
	};
	if _rc==0 {;
		matrix A = e(b); svmat double A; scalar b_psmbc1 = A1[1]; drop A1;
	};
	
	*Calculate one measure of overlap;
	gen within=.; comsup2 `TREATMENT' within;
	sum `TREATMENT' if within; scalar N_over1 = r(N); scalar overlap1 = N_over1/N_orig;
	sum `TREATMENT' if within==1 & `TREATMENT'==1; scalar N_T_over1 = r(N);
	sum `TREATMENT' if within==1 & `TREATMENT'==0; scalar N_C_over1 = r(N);
	
	*Calculate another measure of overlap;
	comsup `TREATMENT';
	sum `TREATMENT'; scalar N_over2 = r(N); scalar overlap2 = N_over2/N_orig;
	sum `TREATMENT' if `TREATMENT'==1; scalar N_T_over2 = r(N);
	sum `TREATMENT' if `TREATMENT'==0; scalar N_C_over2 = r(N);
	
	//STORE THE OUTPUT;
	post results
		(`j') (b_lr) (b_oaxaca) (b_dr) (b_kern_u) (b_ipw2) (b_psm1) (b_psmbc1) (band_kern_u)
		(frac_treated) (N_orig) (N_over1) (N_over2) (N_T_orig) (N_C_orig) (N_T_over1) (N_C_over1) (N_T_over2) (N_C_over2) (overlap1) (overlap2);
	};
	timer off 1;
	timer list 1;
};

postclose results;
clear all;
