*** This is the control file for comparing the performance of       ***
***    various nonexperimental estimators of the ATT (under CIA)    ***
*** Simulation design (pla/str): placebo (Huber et al. 2013)        ***
*** Nonexperimental dataset: CPS                                    ***
*** Outcome variable (re78/u78): earnings                           ***
*** Version of the dataset (DW/ST): Smith & Todd                    ***
*** Choice of control vars (bal/sim): DW (based on balancing tests) ***
*** Target group from NSW (ctr/exp): control group                  ***
*** First version: 30/04/2018                                       ***
*** This version:  24/05/2018                                       ***
*** Authors: Arun Advani, Toru Kitagawa and Tymon Sloczynski        ***
*** Written for Stata 11                                            ***


//all the usual setup stuff
*version control
version 11

*setup standard things
capture clear all
capture log close
capture graph drop _all

*set processors 1

local NAME "placebo_TS_20180524"
set seed 2018052404

*allocate memory
set mem 300m

*get rid of scroll requirement
set more off

*open log
log using `NAME', replace

*install necessary programs
capture {
	ssc install fairlie, replace
	ssc install nnmatch, replace
	ssc install oaxaca, replace
	ssc install psmatch2, replace
	ssc install moremata, replace
}


//setup locals
local LID			301 //ID of sample with lowest ID number
local HID			400 //ID of sample with highest ID number
local NREPS			1000 //number of EMCS replications per sample

local NOBS			1900
local PRTREAT		.05 //the expected proportion of treated in each sample
local SELECT		1 //the magnitude of selection (see \lambda in Huber et al. 2010, p. 23)

local FOLDER		samples
local ESTIMATES		2_estimates

local OUTCOME       re78
local TREATMENT     treated
local COVARIATES    age age2 age3 educ educ2 nodegree married black hispanic re74 re75 u74 u75 re74ed
local COV1			age
local COV2			age2
local COV3			age3
local COV4			educ
local COV5			educ2
local COV6			nodegree
local COV7			married
local COV8			black
local COV9			hispanic
local COV10			re74
local COV11			re75
local COV12			u74
local COV13			u75
local COV14			re74ed
local NCOVS			15 //number of COVARIATES + 1

*************************



#delimit ;

do common_programs;

forvalues j=`LID'/`HID' {;
	
	quietly {;
	capture erase `NAME'_all_samples_`j'.dta;
	
	quietly postfile results
		b_lr b_oaxaca b_dr b_kern_u b_ipw2 b_psm1 b_psmbc1 ave_effect frac_treated N_orig N_over1 N_over2 N_T_orig N_C_orig N_T_over1 N_C_over1 N_T_over2 N_C_over2 overlap1 overlap2
		using `NAME'_results_`j', replace;
	
	timer on 2;
	
	use "../`ESTIMATES'", clear;
	sum band_kern_u if sample==`j';
	local bw = r(sum);
	
	use "../`FOLDER'/sample`j'", clear;
	
	gen u74 = 1 if re74==0;	replace u74 = 0 if re74!=0;
	gen u75 = 1 if re75==0;	replace u75 = 0 if re75!=0;
	gen u78 = 1 if re78==0; replace u78 = 0 if re78!=0;
	
	gen age2 = age^2; gen age3 = age^3;
	gen educ2 = educ^2;
	gen re742 = re74^2; gen re752 = re75^2;
	gen u74h = u74*hispanic; gen re74ed = educ*re74;
	
	*estimate the 'true' propensity score;
	logit `TREATMENT' `COVARIATES', asis tolerance(1e-4) iterate(5000);
	predict double pscore, xb;
	
	*control the degree of selection into treatment;
	replace pscore = pscore*`SELECT';
	
	*drop the treated;
	drop if `TREATMENT'==1;
	
	*save this file -- this is our 'population';
	save `NAME'_population, replace;
	capture drop _all;
	};
	
	forvalues i=1/`NREPS' {;
		quietly {;
		timer on 1;
		
		*******************************************DRAW A SAMPLE************************************************;
		
		use `NAME'_population;
		
		postfile MC_sample `OUTCOME' `COV1' `COV2' `COV3' `COV4' `COV5' `COV6' `COV7' `COV8' `COV9' `COV10' `COV11' `COV12' `COV13' `COV14' pscore
			using `NAME'_sample, replace;
		
		forvalues x=1/`NOBS' {;
			gen random = uniform(); sort random; drop random;
			local k=1;
			while `k'<`NCOVS' {;
				scalar v_`COV`k'' = `COV`k''[1];
				local k=`k'+1;
			};
			scalar v_`OUTCOME' = `OUTCOME'[1];
			scalar v_pscore = pscore[1];
			post MC_sample	(v_`OUTCOME') (v_`COV1') (v_`COV2') (v_`COV3') (v_`COV4') (v_`COV5') (v_`COV6') (v_`COV7') (v_`COV8') (v_`COV9') (v_`COV10')
							(v_`COV11') (v_`COV12') (v_`COV13') (v_`COV14') (v_pscore);
		};
		
		postclose MC_sample; capture drop _all;
		
		*************************************SIMULATE A PLACEBO TREATMENT***************************************;
		
		use `NAME'_sample;
		
		*draw an error from the logistic distribution (mean=0, sd=1);
		gen u_logistic = -1*(sqrt(3)/_pi)*ln((1/uniform())-1);
	
		replace pscore=pscore+u_logistic;

		*impose the expected proportion of treated;
		local PRTREAT2 = 100-(100*`PRTREAT');
		if `SELECT'==0 scalar lambda = (sqrt(3)/_pi)*ln(`PRTREAT'/(1-`PRTREAT'));
		else if `SELECT'!=0 {;
			centile pscore, c(`PRTREAT2');
			scalar lambda = -1*r(c_1);
		};

		*generate the treatment variable;
		*gen `TREATMENT' = 1 if pscore+u_logistic+lambda>0;
		gen `TREATMENT' = 1 if pscore+lambda>0;
		replace `TREATMENT' = 0 if `TREATMENT'==.;
		
		drop pscore u_logistic;
		
		save `NAME'_sample, replace;

		local state = c(seed);
		di c(seed);
		
		*****************************************RUN THE ESTIMATION*********************************************;

		*record the proportion of treated in this sample (and other basic measures);
		sum `TREATMENT'; scalar frac_treated = r(mean); scalar N_orig = r(N);
		sum `TREATMENT' if `TREATMENT'==1; scalar N_T_orig = r(N);
		sum `TREATMENT' if `TREATMENT'==0; scalar N_C_orig = r(N);
		
		*estimate propensity scores;
		logit `TREATMENT' `COVARIATES', asis tolerance(1e-4) iterate(5000);
		predict double logps;
		
		*this is required by kernel matching;
		sort `TREATMENT';

		*linear regression;
		reg `OUTCOME' `TREATMENT' `COVARIATES';
		scalar b_lr = _b[`TREATMENT'];

		*linear Oaxaca-Blinder;
		capture oaxaca `OUTCOME' `COVARIATES', by(`TREATMENT') weight(1);
		if _rc==0 {;
			scalar b_oaxaca = -1*_b[unexplained];
		};
		if _rc!=0 {;
			reg `OUTCOME' `COVARIATES' if `TREATMENT'==0;
			predict pred; gen diff = `OUTCOME' - pred;
			sum diff if `TREATMENT'==1; scalar diff_mean = r(mean);
			oaxaca `OUTCOME' `COVARIATES', by(`TREATMENT') weight(1) relax;
			if round(-1*_b[unexplained],.0001)==round(diff_mean,.0001) {;
				scalar b_oaxaca = -1*_b[unexplained];
			};
			else {;
				scalar b_oaxaca = .;
			};
			drop pred diff;
		};		
		
		capture drop A1-A`NCOVS';
		
		*kernel matching (uniform);
		capture psmatch2 `TREATMENT', kernel out(`OUTCOME') k(uniform) p(logps) bw(`bw');
		if _rc==0 {;
			scalar b_kern_u = r(att);
		};
		if _rc!=0 {;
			scalar b_kern_u = .;
		};
		capture drop _*;
		
		*double robust regression;
		capture drobust `OUTCOME' `TREATMENT' "`COVARIATES'" no;
		if _rc==0 {;
			scalar b_dr = drob;
		};
		if _rc!=0 {;
			scalar b_dr = .;
		};

		*inverse probability weighting;
		capture ipw `OUTCOME' `TREATMENT' `COVARIATES';
		if _rc==0 {;
			scalar b_ipw2 = ipw2;
		};
		if _rc!=0 {;
			scalar b_ipw2 = .;
		};
		
		*matching on the estimated propensity score;
		capture nnmatch `OUTCOME' `TREATMENT' logps, tc(att) m(1);
		if _rc!=0 {;
			scalar b_psm1 = .;
		};
		if _rc==0 {;
			matrix A = e(b); svmat double A; scalar b_psm1 = A1[1]; drop A1;
		};

		*bias-corrected matching on the estimated propensity score;
		capture nnmatch `OUTCOME' `TREATMENT' logps, tc(att) m(1) biasadj(`COVARIATES');
		if _rc!=0 {;
			scalar b_psmbc1 = .;
		};
		if _rc==0 {;
			matrix A = e(b); svmat double A; scalar b_psmbc1 = A1[1]; drop A1;
		};
		
		*The average treatment effect is always zero by construction;
		scalar ave_effect = 0;
		
		*Calculate one measure of overlap;
		gen within=.; comsup2 `TREATMENT' within;
		sum `TREATMENT' if within; scalar N_over1 = r(N); scalar overlap1 = N_over1/N_orig;
		sum `TREATMENT' if within==1 & `TREATMENT'==1; scalar N_T_over1 = r(N);
		sum `TREATMENT' if within==1 & `TREATMENT'==0; scalar N_C_over1 = r(N);
		
		*Calculate another measure of overlap;
		comsup `TREATMENT';
		sum `TREATMENT'; scalar N_over2 = r(N); scalar overlap2 = N_over2/N_orig;
		sum `TREATMENT' if `TREATMENT'==1; scalar N_T_over2 = r(N);
		sum `TREATMENT' if `TREATMENT'==0; scalar N_C_over2 = r(N);
		
		set seed `state';	
		
		//STORE THE OUTPUT;
		post results
			(b_lr) (b_oaxaca) (b_dr) (b_kern_u) (b_ipw2) (b_psm1) (b_psmbc1)
			(ave_effect) (frac_treated) (N_orig) (N_over1) (N_over2) (N_T_orig) (N_C_orig) (N_T_over1) (N_C_over1) (N_T_over2) (N_C_over2) (overlap1) (overlap2);
		
		timer off 1;
		noisily timer list 1;
		
		capture drop _all;
		
		capture use `NAME'_all_samples_`j', clear;
		append using `NAME'_sample;
		keep `OUTCOME' `TREATMENT' age educ nodegree married black hispanic re74 re75;
		save `NAME'_all_samples_`j', replace;
		};
	};
	timer clear 1;
	
	timer off 2;
	noisily display in green "Finished with sample " in yel `j';
	noisily timer list 2;
	noisily display _newline(1);
	
	postclose results;
	capture drop _all;
	erase `NAME'_sample.dta;
	erase `NAME'_population.dta;
};

log close;
