*** This is the control file for comparing the performance of       ***
***    various nonexperimental estimators of the ATT (under CIA)    ***
*** Simulation design: nonparametric bootstrap                      ***
*** Nonexperimental dataset: CPS                                    ***
*** Outcome variable (re78/u78): earnings                           ***
*** Version of the dataset (DW/ST): Dehejia & Wahba                 ***
*** Choice of control vars (bal/sim): DW (based on balancing tests) ***
*** Target group from NSW (ctr/exp): experimental group             ***
*** First version: 22/05/2018                                       ***
*** This version:  24/05/2018                                       ***
*** Authors: Arun Advani, Toru Kitagawa and Tymon Sloczynski        ***
*** Written for Stata 11                                            ***


//all the usual setup stuff
*version control
version 11

*setup standard things
capture clear all
capture log close
capture graph drop _all

*set processors 1

local NAME "bootstrap_TS_20180524"
set seed 2018052406

*allocate memory
set mem 300m

*get rid of scroll requirement
set more off

*open log
log using `NAME', replace

*install necessary programs
capture {
	ssc install fairlie, replace
	ssc install nnmatch, replace
	ssc install oaxaca, replace
	ssc install psmatch2, replace
	ssc install moremata, replace
}


//setup locals
local LID			501 //ID of sample with lowest ID number
local HID			600 //ID of sample with highest ID number
local NREPS			500 //number of EMCS replications per sample

local NOBS			2000

local FOLDER		samples
local ESTIMATES		2_estimates

local OUTCOME       re78
local TREATMENT     treated
local COVARIATES    age age2 age3 educ educ2 nodegree married black hispanic re74 re75 u74 u75 re74ed
local COV1			age
local COV2			age2
local COV3			age3
local COV4			educ
local COV5			educ2
local COV6			nodegree
local COV7			married
local COV8			black
local COV9			hispanic
local COV10			re74
local COV11			re75
local COV12			u74
local COV13			u75
local COV14			re74ed
local NCOVS			15 //number of COVARIATES + 1

*************************



#delimit ;

do common_programs;

forvalues j=`LID'/`HID' {;
	
	quietly {;
	
	quietly postfile results b_lr b_oaxaca b_dr b_kern_u b_ipw2 b_psm1 b_psmbc1 using `NAME'_results_`j', replace;
	
	timer on 2;
	
	use "../`ESTIMATES'", clear;
	sum band_kern_u if sample==`j';
	local bw = r(sum);
	
	use "../`FOLDER'/sample`j'", clear;
	
	gen u74 = 1 if re74==0;	replace u74 = 0 if re74!=0;
	gen u75 = 1 if re75==0;	replace u75 = 0 if re75!=0;
	gen u78 = 1 if re78==0; replace u78 = 0 if re78!=0;
	
	gen age2 = age^2; gen age3 = age^3;
	gen educ2 = educ^2;
	gen re742 = re74^2; gen re752 = re75^2;
	gen u74h = u74*hispanic; gen re74ed = educ*re74;
	
	save `NAME'_population, replace;
	capture drop _all;
	};
	
	forvalues i=1/`NREPS' {;
		quietly {;
		timer on 1;
		
		use `NAME'_population;
		
		postfile MC_sample `OUTCOME' `TREATMENT' `COV1' `COV2' `COV3' `COV4' `COV5' `COV6' `COV7' `COV8' `COV9' `COV10' `COV11' `COV12' `COV13' `COV14'
			using `NAME'_sample, replace;
		
		forvalues x=1/`NOBS' {;
			gen random = uniform(); sort random; drop random;
			local k=1;
			while `k'<`NCOVS' {;
				scalar v_`COV`k'' = `COV`k''[1];
				local k=`k'+1;
			};
			scalar v_`OUTCOME' = `OUTCOME'[1];
			scalar v_`TREATMENT' = `TREATMENT'[1];
			post MC_sample	(v_`OUTCOME') (v_`TREATMENT') (v_`COV1') (v_`COV2') (v_`COV3') (v_`COV4') (v_`COV5') (v_`COV6') (v_`COV7') (v_`COV8') (v_`COV9') (v_`COV10')
							(v_`COV11') (v_`COV12') (v_`COV13') (v_`COV14');
		};
		
		postclose MC_sample; capture drop _all; use `NAME'_sample;

		*estimate propensity scores;
		logit `TREATMENT' `COVARIATES', asis tolerance(1e-4) iterate(1000);
		predict double logps;
		
		*this is required by kernel matching;
		sort `TREATMENT';

		*linear regression;
		reg `OUTCOME' `TREATMENT' `COVARIATES';
		scalar b_lr = _b[`TREATMENT'];

		*linear Oaxaca-Blinder;
		capture oaxaca `OUTCOME' `COVARIATES', by(`TREATMENT') weight(1);
		if _rc==0 {;
			scalar b_oaxaca = -1*_b[unexplained];
		};
		if _rc!=0 {;
			reg `OUTCOME' `COVARIATES' if `TREATMENT'==0;
			predict pred; gen diff = `OUTCOME' - pred;
			sum diff if `TREATMENT'==1; scalar diff_mean = r(mean);
			oaxaca `OUTCOME' `COVARIATES', by(`TREATMENT') weight(1) relax;
			if round(-1*_b[unexplained],.0001)==round(diff_mean,.0001) {;
				scalar b_oaxaca = -1*_b[unexplained];
			};
			else {;
				scalar b_oaxaca = .;
			};
			drop pred diff;
		};
		capture drop A1-A`NCOVS';
		
		*kernel matching (uniform);
		capture psmatch2 `TREATMENT', kernel out(`OUTCOME') k(uniform) p(logps) bw(`bw');
		if _rc==0 {;
			scalar b_kern_u = r(att);
		};
		if _rc!=0 {;
			scalar b_kern_u = .;
		};
		capture drop _*;
		
		*double robust regression;
		capture drobust `OUTCOME' `TREATMENT' "`COVARIATES'" no;
		if _rc==0 {;
			scalar b_dr = drob;
		};
		if _rc!=0 {;
			scalar b_dr = .;
		};

		*inverse probability weighting;
		capture ipw `OUTCOME' `TREATMENT' `COVARIATES';
		if _rc==0 {;
			scalar b_ipw2 = ipw2;
		};
		if _rc!=0 {;
			scalar b_ipw2 = .;
		};
		
		*matching on the estimated propensity score;
		capture nnmatch `OUTCOME' `TREATMENT' logps, tc(att) m(1);
		if _rc!=0 {;
			scalar b_psm1 = .;
		};
		if _rc==0 {;
			matrix A = e(b); svmat double A; scalar b_psm1 = A1[1]; drop A1;
		};

		*bias-corrected matching on the estimated propensity score;
		capture nnmatch `OUTCOME' `TREATMENT' logps, tc(att) m(1) biasadj(`COVARIATES');
		if _rc!=0 {;
			scalar b_psmbc1 = .;
		};
		if _rc==0 {;
			matrix A = e(b); svmat double A; scalar b_psmbc1 = A1[1]; drop A1;
		};
		
		//STORE THE OUTPUT;
		post results (b_lr) (b_oaxaca) (b_dr) (b_kern_u) (b_ipw2) (b_psm1) (b_psmbc1);
		
		timer off 1;
		noisily timer list 1;
		
		capture drop _all;
		};
	};
	timer clear 1;
	
	timer off 2;
	noisily display in green "Finished with sample " in yel `j';
	noisily timer list 2;
	noisily display _newline(1);
	
	postclose results;
	capture drop _all;
	erase `NAME'_sample.dta;
	erase `NAME'_population.dta;
};

log close;
